@@ -1,16 +1,17 @@
 /* ptst.c:  build a .prc from a pile of files.
  *
  * (c) 1996, Dionne & Associates
+ * (c) 1997, The Silver Hammer Group Ltd.
  * This is Free Software, under the GNU Public Licence v2 or greater.
  */
 
 #include <unistd.h>
 #include <stdio.h>
 #include <fcntl.h>
-#include <malloc.h>
+#include <stdlib.h>
 #include "prc.h"
 
-#ifdef CYGWIN32
+#ifdef __CYGWIN32__
 #define O_PLATFORM O_BINARY
 #else
 #define O_PLATFORM 0
@@ -24,20 +25,45 @@ main(int argc, char *argv[])
   char *sd;
   char sname[5];
   int slen;
+  int opt;
+  char *type = "appl";
+  int optind = 1;
 
   if (argc < 5) {
-    fprintf(stderr, "Usage: %s fname.prc 'App Name' apid resource resource ...\n", argv[0]);
+    fprintf(stderr, "Usage: %s [-Ll] [-t type] fname.prc 'App Name' apid resource resource ...\n", argv[0]);
     exit (1);
   }
 
-  if (!(pf = openw_pfd(argv[2], argv[3], argv[1]))) {
+  while (optind < argc && argv[optind][0] == '-') {
+    opt = argv[optind++][1];
+
+    switch (opt) {
+
+    case 'l':
+      type = "GLib";
+      break;
+    case 'L':
+      type = "libr";
+      break;
+    case 't':
+      type = argv[optind++];
+      break;
+    case '?':
+    default :
+      fprintf(stderr, "%s Unknown option\n", argv[0]);
+      exit (2);
+      break;
+    }
+  }
+
+  if (!(pf = openw_pfd(argv[optind + 1], argv[optind + 2], argv[optind], type))) {
     printf ("can't open prc file\n");
     exit(2);
   }
 
   /* the .prc write routines write the resources in reverse order! */
 
-  for (i=argc-1; i > 3; i--) {
+  for (i=argc-1; i > optind + 2; i--) {
     if ((fd = open(argv[i], O_PLATFORM | O_RDONLY)) < 1) {
       fprintf(stderr, "Can't open resource file %s\n",argv[i]);
       exit(3);
