@@ -21,16 +21,24 @@
 #include "pilrc.h"
 #include "bitmap.h"
 
+
+#define idPalmOSReservedMin 10000
+
 /*-----------------------------------------------------------------------------
 |	 Globals
 -------------------------------------------------------------WESC------------*/
 /* Are we PilRCUI -- this was an attempt to merge pilrc and pilrcui */
 BOOL vfWinGUI;
-
+/* Quiet ouptput */
+BOOL vfQuiet;
 /* Translations */
 char *szLanguage;
 TE *pteFirst;
 
+/* Warning about duplicates in forms and menus */
+/* if == 2 then we warn about duplicate labels too */
+BOOL vfCheckDupes;
+
 /* Form globals */
 RCFORM form;	
 #define cobjMax 256
@@ -48,6 +56,17 @@ RCMENUITEM rgmi[imiMax];
 RCMENUPULLDOWN rgmpd[impdMax];
 int imiMac;	
 int idMenu;
+#define iidMenuMax 128
+int iidMenuMac;
+int rgidMenu[iidMenuMax];
+#define iidStringMax 512
+int iidStringMac;
+int rgidString[iidStringMax];
+#define iidAlertMax 512
+int iidAlertMac;
+int rgidAlert[iidAlertMax];
+
+
 
 /* Symbol table */
 SYM *psymFirst;
@@ -59,9 +78,12 @@ TOK tok;
 int iline;
 char szLine[4096];
 
+
 /* Rect for Prev* keywords */
 RCRECT rcPrev;
 
+
+
 /*-----------------------------------------------------------------------------
 |	AddSym
 |	
@@ -288,19 +310,19 @@ VOID GetExpectLt(TOK *ptok, LT lt, char *szErr)
 		if (szErr == NULL)
 			{
 			if (lt == ltId)
-				ErrorLine("Syntax error : expecting identifier");
+				ErrorLine2("Syntax error : expecting identifier, got", ptok->lex.szId);
 			else if (lt == ltStr)
-				ErrorLine("Syntax error : expecting string");			
+				ErrorLine2("Syntax error : expecting string, got", ptok->lex.szId);			
 			else if (lt == ltConst)
-				ErrorLine("Syntax error : expecting constant");
+				ErrorLine2("Syntax error : expecting constant, got", ptok->lex.szId);
 			else
-				ErrorLine("syntax error");
+				ErrorLine2("syntax error: ", ptok->lex.szId);
 			}
 		else
 			{
 			char szT[128];
-			sprintf(szT, "expecting: %s", szErr);
-			ErrorLine(szT);
+			sprintf(szT, "expecting: %s, got", szErr);
+			ErrorLine2(szT, ptok->lex.szId);
 			}
 		}
 	}
@@ -319,8 +341,8 @@ VOID GetExpectRw(RW rw)
 	if (tok.rw != rw)
 		{
 		char szErr[64];
-		sprintf(szErr, "%s expected", PchFromRw(rw, fTrue));
-		ErrorLine(szErr);
+		sprintf(szErr, "%s expected, got", PchFromRw(rw, fTrue));
+		ErrorLine2(szErr, tok.lex.szId);
 		}
 	}
 
@@ -375,13 +397,15 @@ char *PchGetId(char *szErr)
 |	WGetConst
 |	
 |		Get an integer constant or one of the Prev* reserved words, returning
-|	value.
-|	
+|	valu	e.
+|		
 -------------------------------------------------------------WESC------------*/
 int WGetConst(char *szErr)
 	{
+	char sz[256];
+
 	if (!FGetTok(&tok))
-		ErrorLine(szErr);
+		ErrorLine("unexpected end of file");
 	switch (tok.rw)
 		{
 	default:
@@ -391,13 +415,18 @@ int WGetConst(char *szErr)
 
 			psym = PsymLookup(tok.lex.szId);
 			if (psym == NULL)
-				ErrorLine("Unknown symbol");
+				{
+				sprintf(sz, "Expecting %s, got unknown symbol:", szErr);
+				ErrorLine2(sz, tok.lex.szId);
+				}
 			return psym->wVal;
 			}
 		if (tok.lex.lt != ltConst)
-			ErrorLine(szErr);
+			{
+			sprintf(sz, "%s expected, got", szErr);
+			ErrorLine2(sz, tok.lex.szId);
+			}
 		return tok.lex.val;
-		break;
 	case rwPrevLeft:
 		return  rcPrev.topLeft.x;
 	case rwPrevRight:
@@ -720,10 +749,7 @@ VOID DoCheckGrif(int grif, int ifP)
 	{
 	if (!(grif & ifP))
 		{
-		char szT[256];
- 
-		sprintf(szT, "%s unexpected", tok.lex.szId);
-		ErrorLine(szT);
+		ErrorLine2(tok.lex.szId, "unexpected");
 		}
 	}
 
@@ -734,11 +760,16 @@ VOID DoCheckGrif(int grif, int ifP)
 -------------------------------------------------------------WESC------------*/
 int WGetId(char *szErr)
 	{
+	int w;
+
 	if (!FGetTok(&tok))
 		return 0;
 	if (tok.rw != rwId)
 		UngetTok();
-	return WGetConstEx(szErr);
+	w = WGetConstEx(szErr);
+	if (w >= idPalmOSReservedMin)
+		ErrorLine("ID conflicts with PalmOS reserved ID range (valid values: 0-9999)");
+	return w;
 	}
 
 #define CheckGrif(ifP) do {DoCheckGrif(pitm->grif, ifP); pitm->grifOut |= ifP;} while (0);
@@ -803,12 +834,12 @@ void ParseItm(ITM *pitm, int grif, int grif2)
 	if (grif & ifId)
 		{
 		pitm->grifOut |= ifId;
-		pitm->id = WGetId("item id");
+		pitm->id = WGetId("ItemId");
 		}
 	if (grif & ifListId)
 		{
 		pitm->grifOut |= ifListId;
-		pitm->listid = WGetConstEx("list id");
+		pitm->listid = WGetConstEx("ListId");
 		}
 
 	fAt = fFalse;
@@ -857,11 +888,11 @@ void ParseItm(ITM *pitm, int grif, int grif2)
 			break;
 		case rwGroup:
 			CheckGrif(ifGroup);
-			pitm->group = WGetConstEx("Group Id"); 
+			pitm->group = WGetConstEx("GroupId"); 
 			break;
 		case rwFont:	/* Add rwBoldFont, rwLargeFont? */
 			CheckGrif(ifFont);
-			pitm->font = WGetConstEx("Font Id");
+			pitm->font = WGetConstEx("FontId");
 			break;
 		case rwFrame:					/* BUG! other frame types */
 		case rwNoFrame:
@@ -897,7 +928,7 @@ void ParseItm(ITM *pitm, int grif, int grif2)
 			break;
 		case rwMaxChars:
 			CheckGrif(ifMaxChars);
-			pitm->maxChars = WGetConstEx("max chars");
+			pitm->maxChars = WGetConstEx("MaxChars");
 			break;
 		case rwLeftAlign:	/* default? */
 			CheckGrif(ifAlign);
@@ -913,11 +944,11 @@ void ParseItm(ITM *pitm, int grif, int grif2)
 			break;
 		case rwVisibleItems:
 			CheckGrif(ifCvis);
-			pitm->cvis = WGetConstEx("visible items");
+			pitm->cvis = WGetConstEx("VisibleItems");
 			break;
 		case rwBitmap:
 			CheckGrif(ifBitmap);
-			pitm->rscID = WGetConstEx("bitmap id");
+			pitm->rscID = WGetConstEx("BitmapId");
 			break;
 		case rwModal:
 			CheckGrif(ifModal);
@@ -929,27 +960,27 @@ void ParseItm(ITM *pitm, int grif, int grif2)
 			break;
 		case rwHelpId:
 			CheckGrif(ifHelpId);
-			pitm->helpId = WGetConstEx("help id");
+			pitm->helpId = WGetConstEx("HelpId");
 			break;
 		case rwDefaultBtnId:
 			CheckGrif(ifDefaultBtnId);
-			pitm->defaultBtnId = WGetConstEx("default button id");
+			pitm->defaultBtnId = WGetConstEx("DefaultButtonId");
 			break;
 		case rwMenuId:
 			CheckGrif(ifDefaultBtnId);
-			pitm->menuId = WGetConstEx("menu id");
+			pitm->menuId = WGetConstEx("MenuId");
 			break;
 		case rwNumColumns:
 			CheckGrif2(if2NumColumns);
-			pitm->numColumns = WGetConstEx("num columns");
+			pitm->numColumns = WGetConstEx("NumColumns");
 			break;
 		case rwNumRows:
 			CheckGrif2(if2NumRows);
-			pitm->numRows = WGetConstEx("num rows");
+			pitm->numRows = WGetConstEx("NumRows");
 			break;
 		case rwColumnWidths:
 			for (icol = 0; icol < pitm->numColumns; icol++)
-				pitm->rgdxcol[icol] = WGetConstEx("column width"); 
+				pitm->rgdxcol[icol] = WGetConstEx("ColumnWidth"); 
 			break;
 			}
 		}			
@@ -1245,7 +1276,32 @@ void DumpForm()
 		}
 	CloseOutput();
 	}
-		
+
+BOOL FIdFormObject(FormObjectKind kind, BOOL fIncludeLabels)
+	{	
+	switch (kind)
+		{
+	case frmFieldObj:
+	case frmControlObj:
+	case frmListObj:
+	case frmTableObj:
+	case frmTitleObj:
+	
+	case frmGadgetObj:
+		return fTrue;
+	case frmLabelObj:
+		return fIncludeLabels;
+	case frmPopupObj:
+	case frmLineObj:
+	case frmFrameObj:
+	case frmRectangleObj:
+	case frmGraffitiStateObj:	
+	case frmBitmapObj:
+		return fFalse;
+		}
+	Assert(fFalse);
+	return fFalse;
+	}		
 	
 /*-----------------------------------------------------------------------------
 |	AddObject
@@ -1254,8 +1310,22 @@ void DumpForm()
 -------------------------------------------------------------WESC------------*/
 VOID AddObject(RCFORMOBJECT *pobj, FormObjectKind kind)
 	{
+	int iobj;
+
 	if (form.numObjects > cobjMax)
 		ErrorLine("Too many objects in current form");
+	if (vfCheckDupes && FIdFormObject(kind, fTrue))
+		{
+		for (iobj = 0; iobj < form.numObjects; iobj++)
+			{
+			if (FIdFormObject(rglt[iobj].objectType, vfCheckDupes == 2) && 
+				pobj->control->id == rglt[iobj].u.object.control->id)
+				{
+				WarningLine("Duplicate form object id (error is on the previous line)");
+				break;
+				}
+			}
+		}
 	rglt[form.numObjects].objectType = kind;	
 	rglt[form.numObjects].u.object = *pobj;
 	form.numObjects++;
@@ -1311,6 +1381,13 @@ void FreeRgfrm()
 	ifrmMac = 0;
 	}
 
+void FreeAll()
+	{
+	FreeRgfrm();
+	iidMenuMac = 0;
+	iidAlertMac = 0;
+	iidStringMac = 0;
+	}
 
 /*-----------------------------------------------------------------------------
 |	FParseObjects
@@ -1470,7 +1547,7 @@ Control:
 				fok = frmTableObj;
 				break;
 			default:
-				 ErrorLine("Unknown token");
+				 ErrorLine2("Unknown token:", tok.lex.szId);
 				 break;
 			}
 		if (fok != (FormObjectKind) -1)
@@ -1514,15 +1591,28 @@ BOOL FParseForm()
 
 	GetExpectRw(rwBegin);
 	f = FParseObjects();
-#ifdef WINGUI
+//#ifdef WINGUI
 	if (ifrmMac > ifrmMax)
 		Error("Too many forms!");
+	if (vfCheckDupes)
+		{
+		int ifrm;
+
+		for (ifrm = 0; ifrm < ifrmMac; ifrm++)
+			{
+			if (rgfrm[ifrm].form.formId == form.formId)
+				{
+				ErrorLine("Duplicate Form Resource IDs");
+				break;
+				}
+			}
+		}
 	rgfrm[ifrmMac].form = form;
 	rgfrm[ifrmMac].rglt = calloc(sizeof(RCFORMOBJLIST), form.numObjects);
 	memcpy(rgfrm[ifrmMac].rglt, rglt, sizeof(RCFORMOBJLIST) * form.numObjects);
 	/* clone pointers?  yuck */
 	ifrmMac++;
-#endif
+//#endif
 	return f;
 	}
 										
@@ -1609,6 +1699,7 @@ VOID DumpMenu()
 			}
 		}
 	CloseOutput();
+	
 	}
 
 	
@@ -1723,6 +1814,23 @@ BOOL FParsePullDown()
 						UngetTok();
 					}
 				}
+				if (vfCheckDupes)
+					{
+					int imi;
+
+					for (imi = 0; imi < imiMac; imi++)
+						{
+						if (strcmp(rgmi[imi].itemStr, "-") != 0)
+							{
+							if (mi.id == rgmi[imi].id)
+								{
+								WarningLine("Duplicate menu item id");
+								break;
+								}
+							}
+						}
+					}
+
 			if (imiMac > imiMax)
 				ErrorLine("Too many menu items");
 			rgmi[imiMac++] = mi;
@@ -1748,6 +1856,22 @@ BOOL FParseMenu()
 	imiMac = 0;
 
 	idMenu = WGetId("MenuId");
+
+	if (vfCheckDupes)
+		{
+		int iid;
+
+		for (iid = 0; iid < iidMenuMac; iid++)
+			{
+			if (rgidMenu[iid] == idMenu)
+				{
+				ErrorLine("Duplicate Menu Resource ID");
+				}
+			}
+		}
+
+	rgidMenu[iidMenuMac++] = idMenu;
+
 	menu.curItem = -1;	/* all resource I've seen have this set */
 	GetExpectRw(rwBegin);
 	while (FGetTok(&tok))
@@ -1755,7 +1879,7 @@ BOOL FParseMenu()
 		switch (tok.rw)
 			{
 		default:
-			ErrorLine("Unknown identifier");
+			ErrorLine2("Unknown identifier:", tok.lex.szId);
 			break;			
 		case rwPullDown:
 			FParsePullDown();
@@ -1806,7 +1930,23 @@ void ParseDumpAlert()
 	memset(&itm, 0, sizeof(itm));
 	pchTitle = rgbZero;
 	pchMessage = rgbZero;
-	idAlert = WGetId("alert id");
+	idAlert = WGetId("AlertId");
+
+	if (vfCheckDupes)
+		{
+		int iid;
+
+		for (iid = 0; iid < iidAlertMac; iid++)
+			{
+			if (rgidAlert[iid] == idAlert)
+				{
+				ErrorLine("Duplicate Alert Resource ID");
+				}
+			}
+		}
+	if (iidAlertMac < iidAlertMax)
+		rgidAlert[iidAlertMac++] = idAlert;
+
 	
 	while (FGetTok(&tok))
 		{
@@ -1822,7 +1962,7 @@ void ParseDumpAlert()
 			at.alertType = tok.rw-rwInformation;
 			break;
 		case rwHelpId:
-			at.helpRscID = WGetConstEx("help id");
+			at.helpRscID = WGetConstEx("HelpId");
 			break;
 		case rwBegin:
 			while (FGetTok(&tok))
@@ -1889,6 +2029,23 @@ void ParseDumpString()
 	char *pchString;
 
 	id = WGetId("String ResourceId");
+
+	if (vfCheckDupes)
+		{
+		int iid;
+
+		for (iid = 0; iid < iidMenuMac; iid++)
+			{
+			if (rgidString[iid] == id)
+				{
+				ErrorLine("Duplicate String Resource ID");
+				}
+			}
+		}
+	if (iidStringMac < iidStringMax)
+		rgidString[iidStringMac++] = id;
+
+
 	pchString = PchGetSzMultiLine("String Text");
 	OpenOutput("tSTR", id);
 	DumpBytes(pchString, strlen(pchString)+1);
@@ -1939,7 +2096,7 @@ void ParseDumpApplicationIconName()
 	int id;
 	char *pchString;
 
-	id = WGetId("application icon name resource id");
+	id = WGetId("Application IconName ResourceId");
 	pchString = PchGetSz("Icon Name Text");
 	OpenOutput("tAIN", id);
 	DumpBytes(pchString, strlen(pchString)+1);
@@ -1956,7 +2113,7 @@ void ParseDumpApplication()
 	int id;
 	char *pchString;
 
-	id = WGetId("appl resource id");
+	id = WGetId("APPL ResourceId");
 	pchString = PchGetSz("APPL");
 	if (strlen(pchString) != 4)
 		Error("APPL resource must be 4 chars");
@@ -2055,7 +2212,7 @@ void ParseCInclude( char *szIncludeFile )
 			GetExpectLt(&tok, ltId, "identifier");
 			strcpy(szId, tok.lex.szId);
 GetVal:
-			wIdVal = WGetConstEx("constant");
+			wIdVal = WGetConstEx("Constant");
 			AddSym(szId, wIdVal);
 			break;
 		case ltId:
@@ -2068,7 +2225,7 @@ GetVal:
 			NextLine();
 			break;			
 		default:
-			ErrorLine("identifier expected");
+			ErrorLine("Identifier expected");
 			break;
 			}
 		}	
@@ -2137,7 +2294,7 @@ void ParseJavaInclude( char *szIncludeFile )
 			strcpy(szId, tok.lex.szId);
 			GetExpectLt(&tok, ltAssign, "[public] [static] [final] [short|int] Name = number;");
 
-			wIdVal = WGetConstEx("constant");
+			wIdVal = WGetConstEx("Constant");
 			AddSym(szId, wIdVal);
 			break;
 		case ltSemi:
@@ -2147,7 +2304,7 @@ void ParseJavaInclude( char *szIncludeFile )
 		case ltRBrace:
 			goto endOfClass;
 		default:
-			ErrorLine("identifier definition expected");
+			ErrorLine("Identifier definition expected");
 			break;
 			}
 		}	
@@ -2203,11 +2360,8 @@ void ParseDirectives()
 	/* Add new directives here	*/
 
 	default:
-		{
-		char szErr[64];
-		sprintf(szErr, "directive not supported (try #include)" );
-		ErrorLine(szErr);
-		}
+		ErrorLine2("directive not supported (try #include):", tok.lex.szId);
+		break;
 		}
 
 	}
@@ -2216,10 +2370,11 @@ void ParseDirectives()
 /*-----------------------------------------------------------------------------
 |	ParseFile
 -------------------------------------------------------------WESC------------*/
-void ParseFile(char *szIn, char *szOutDir)
+void ParseFile(char *szIn, char *szOutDir, char *szResFile)
 	{
 	SetOutFileDir(szOutDir);
 	OpenInputFile(szIn);
+	OpenResFile(szResFile);
 	FInitLexer(NULL, fTrue);
 
 	if (!FGetTok(&tok))
@@ -2277,6 +2432,7 @@ void ParseFile(char *szIn, char *szOutDir)
 	fclose(vfhIn);
 	FreeSymTable();
 	FreeTranslations();
+	CloseResFile();
 	}
 
 
@@ -2288,6 +2444,8 @@ void Usage()
 	  "        -I <path>    Search for bitmap and include files in <path>\n"
 	  "                     More than one -I <path> options may be given\n"
 	  "                     The current directory is always searched\n"
+	  "        -R <resfile> Generate JUMP/PilA .res file\n"
+	  "        -q           Less noisy output\n"
 	  );
 
     exit( 1 );
@@ -2301,21 +2459,26 @@ main(int cArg, char *rgszArg[])
 	{
 	char *szOutputPath;
 	char *szInputFile;
+	char *szResFile;
 	int i;
-    
-	printf("pilrc v1.3.  by Wes Cherry (wesc@ricochet.net)\n");
 	
+    	
+	printf("pilrc v1.5.  by Wes Cherry (wesc@ricochet.net)\n");
+
+	vfCheckDupes = 1;	
 	if (cArg < 2)
 		{
 	    Usage();
 		}
 
+	szResFile = NULL;
 	for (i = 1; i < cArg; i++)
 		{
 	    if (rgszArg[i][0] != '-')
 			{
 			break;
 			}
+
 	
 	    if (FSzEqI(rgszArg[i], "-L"))
 			{
@@ -2345,6 +2508,20 @@ main(int cArg, char *rgszArg[])
 			continue;
 			}
 
+	    if (FSzEqI(rgszArg[i], "-R"))
+			{
+			if (i++ == cArg)
+				{
+				Usage();
+				}
+			szResFile = rgszArg[i];
+			continue;
+			}
+	    if (FSzEqI(rgszArg[i], "-q"))
+			{
+			vfQuiet = fTrue;
+			continue;
+			}
 		Usage();
 		}
 
@@ -2371,7 +2548,7 @@ main(int cArg, char *rgszArg[])
 
 	printf("\n");
 
-	ParseFile(szInputFile, szOutputPath);
+	ParseFile(szInputFile, szOutputPath, szResFile);
 
 	return 0;
 	}
