@@ -478,7 +478,7 @@ typedef struct _FRM
 
 extern FRM rgfrm[];
 extern int ifrmMac;
-void FreeRgfrm();
+void FreeAll();
 
 void ErrorLine(char *sz);
 				
