@@ -1,3 +1,5 @@
+#ifndef __UTIL_H__
+#define __UTIL_H__
 
 
 int DxCalcExtent(unsigned char *sz, BOOL fBold);
@@ -6,6 +8,8 @@ VOID Error(char *sz);
 VOID Error2(char *sz1, char *sz2);
 VOID Error3(char *sz1, char *sz2, char *sz3);
 void ErrorLine(char *sz);
+void WarningLine(char *sz);
+void ErrorLine2(char *sz, char *sz2);
 BOOL FSzEqI(char *sz1, char *sz2);
 int WMin(int w1, int w2);
 int WMax(int w1, int w2);
@@ -19,6 +23,8 @@ void EmitL(unsigned long l);
 /*void SwapStruct(void *pv, char *szPic); */
 VOID OpenOutput(char *szBase, int id);
 VOID CloseOutput();
+VOID OpenResFile(char *szFile);
+VOID CloseResFile();
 void DumpBytes(void *pv, int cb);
 void PadWordBoundary();
 int IbOut();
@@ -37,3 +43,5 @@ extern int   totalIncludePaths;
 
 	
 
+
+#endif /*__UTIL_H__*/
\ No newline at end of file
